package com.dayou.controller;

import com.dayou.dto.WorkNodeCommit;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RestController;
import com.dayou.controller.BaseController;
import com.dayou.service.IAssetsService;
import com.dayou.entity.Assets;
import com.dayou.common.RestResponse;
import org.springframework.web.bind.annotation.*;
import com.dayou.common.PullDownModel;
import com.dayou.utils.ConvertUtil;
import com.dayou.utils.HttpKit;
import com.dayou.exception.ErrorCode;
import java.util.Date;
import java.util.List;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.http.MediaType;
import org.springframework.web.multipart.MultipartFile;
/**
 * 资产业务订单表
 *
 * @author GouGengquan
 * @since 2024-03-28
 */
@RestController
@RequestMapping("assets")
@Slf4j
public class AssetsController extends BaseController {
    @Autowired
    private IAssetsService assetsService;

    /**
    * 资产业务订单表列表
    */
    @GetMapping("")
    public RestResponse<Page<Assets>> page(Assets assets, Page page){
        Page<Assets> pages=assetsService.selectPage(page,assets);
        return RestResponse.data(pages);
    }

    /**
     * 资产业务订单表详情
     */
    @GetMapping("/{id}")
    public RestResponse<Assets> detail(@PathVariable Long id){
        Assets xAssets =assetsService.detail(id);
        return RestResponse.data(xAssets);
     }

    /**
     * 资产业务订单新增
     */
    @PostMapping("/add")
    public RestResponse<WorkNodeCommit> save(@RequestBody Assets assets) {
        return RestResponse.data(assetsService.add(assets));
    }

    /**
     * 资产业务订单表更新
     */
    @PutMapping("")
    public RestResponse<Boolean> update(@RequestBody Assets assets) {
        Boolean ret = assetsService.update(assets);
        return RestResponse.data(ret);
    }

    /**
     * 资产业务订单表删除
     */
    @DeleteMapping("/{id}")
    public RestResponse<Boolean> delete(@PathVariable Long id) {
        Boolean ret = assetsService.delete(id);
        return RestResponse.data(ret);
    }


    /**
     * 资产业务订单表下拉列表
     */
    @GetMapping("/simpleAll")
    public RestResponse<List<PullDownModel>> simpleAll(){
        LambdaQueryWrapper<Assets> lambdaQueryWrapper = new LambdaQueryWrapper<Assets>()
        .select(Assets::getId,Assets::getName)
        .eq(Assets::getDeleted,false);
        List<Assets> list= assetsService.list(lambdaQueryWrapper);
        List<PullDownModel> ret=ConvertUtil.copyList(list, PullDownModel.class);
        return RestResponse.data(ret);
    }
}

