package com.dayou.controller;

import com.dayou.common.RestResponse;
import com.dayou.dto.OrderProductionsBag;
import com.dayou.dto.OrderReportDTO;
import com.dayou.dto.ReportDTO;
import com.dayou.service.IMajorProductionService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 类说明：业务产品
 *
 * @author: wucl
 * @since: 2023/12/14
 * created with IntelliJ IDEA.
 */
@RestController
@RequestMapping("businessProduction")
@Slf4j
public class BusinessProductionController {

    @Autowired
    private IMajorProductionService majorProductionService;

    /**
     * 大中型产品包
     */
    @GetMapping("/bag/major/{id}/{reportNo}")
    public RestResponse<OrderProductionsBag> productionBag(@PathVariable("id") Long id,@PathVariable("reportNo") String reportNo){
        OrderProductionsBag bag = majorProductionService.productionBag(id,reportNo);
        return RestResponse.data(bag);
    }

    /**
     * 大中型列表
     */
    @GetMapping("/major/{orderFundId}")
    public RestResponse<List<ReportDTO>> getProductionByOrderFundId(@PathVariable("orderFundId") Long orderFundId){
        List<ReportDTO> productions = majorProductionService.getProductionByOrderFundId(orderFundId);
        return RestResponse.data(productions);
    }

    /**
     * 多个订单报告列表
     * @param claimOrders
     * @return
     */
    @PostMapping("/report/major")
    public RestResponse<List<OrderReportDTO>> getOrderReport(@RequestBody List<OrderReportDTO> claimOrders){
        List<OrderReportDTO> list = majorProductionService.getOrderReport(claimOrders);
        return RestResponse.data(list);
    }

    /**
     * 实收款认领
     * @return
     */
    @PostMapping("/allotRealAmount/major")
    public RestResponse<Boolean> allotRealAmount(@RequestBody List<OrderReportDTO> claimOrders){
        Boolean ret = majorProductionService.allotRealAmount(claimOrders);
        return RestResponse.data(ret);
    }

    /**
     * 实收款分配到产品
     * @return
     */
    @PostMapping("/major/allot")
    public RestResponse<Boolean> allotRealAmountToProduction(@RequestBody OrderReportDTO orderReportDTO){
        Boolean ret = majorProductionService.allotRealAmountToProduction(orderReportDTO);
        return RestResponse.data(ret);
    }

}
