package com.dayou.controller;

import com.dayou.vo.FinanceClaimVO;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RestController;
import com.dayou.controller.BaseController;
import com.dayou.service.IFinanceClaimService;
import com.dayou.entity.FinanceClaim;
import com.dayou.common.RestResponse;
import org.springframework.web.bind.annotation.*;
import com.dayou.utils.ConvertUtil;
import com.dayou.utils.HttpKit;
import com.dayou.exception.ErrorCode;
import java.util.Date;
import java.util.List;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.http.MediaType;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;

/**
 * 实收款认领
 *
 * @author wucl
 * @since 2023-12-08
 */
@RestController
@RequestMapping("financeClaim")
@Slf4j
public class FinanceClaimController extends BaseController {
    @Autowired
    private IFinanceClaimService financeClaimService;

    /**
    * 实收款认领列表
    */
    @GetMapping("")
    public RestResponse<Page<FinanceClaim>> page(FinanceClaim financeClaim, Page page){
        Page<FinanceClaim> pages=financeClaimService.selectPage(page,financeClaim);
        return RestResponse.data(pages);
    }

    /**
     * 实收款认领详情
     */
    @GetMapping("/{id}")
    public RestResponse<FinanceClaim> detail(@PathVariable Long id){
        FinanceClaim xFinanceClaim =financeClaimService.detail(id);
        return RestResponse.data(xFinanceClaim);
     }

    /**
     * 实收款认领新增
     */
    @PostMapping("")
    public RestResponse<Boolean> save(@RequestBody @Valid FinanceClaim financeClaim) {
        Boolean ret = financeClaimService.add(financeClaim);
        return RestResponse.data(ret);
    }

    /**
     * 实收款认领更新
     */
    @PutMapping("")
    public RestResponse<Boolean> update(@RequestBody FinanceClaim financeClaim) {
        Boolean ret = financeClaimService.update(financeClaim);
        return RestResponse.data(ret);
    }

    /**
     * 实收款认领删除
     */
    @DeleteMapping("/{id}")
    public RestResponse<Boolean> delete(@PathVariable Long id) {
        Boolean ret = financeClaimService.delete(id);
        return RestResponse.data(ret);
    }

    /**
     * 某实收款认领列表
     */
    @GetMapping("/list/{id}")
    public RestResponse<List<FinanceClaimVO>> getList(@PathVariable Long id){
        List<FinanceClaimVO> list = financeClaimService.getList(id);
        return RestResponse.data(list);
    }
}

