package com.dayou.controller;

import com.dayou.dto.TaskRecordDTO;
import com.dayou.dto.WorkNodeCommit;
import com.dayou.vo.PersonalVO;
import com.dayou.workflow.annotation.FinishTask;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RestController;
import com.dayou.controller.BaseController;
import com.dayou.service.IPersonalService;
import com.dayou.entity.Personal;
import com.dayou.common.RestResponse;
import org.springframework.web.bind.annotation.*;
import com.dayou.utils.ConvertUtil;
import com.dayou.utils.HttpKit;
import com.dayou.exception.ErrorCode;
import java.util.Date;
import java.util.List;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.http.MediaType;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;

/**
 * 个贷业务订单
 *
 * @author wucl
 * @since 2024-03-01
 */
@RestController
@RequestMapping("personal")
@Slf4j
public class PersonalController extends BaseController {
    @Autowired
    private IPersonalService personalService;

    /**
    * 个贷业务订单列表
    */
    @GetMapping("")
    public RestResponse<Page<PersonalVO>> page(PersonalVO personal, Page page){
        Page<PersonalVO> pages=personalService.selectPage(page,personal);
        return RestResponse.data(pages);
    }

    /**
     * 个贷业务订单详情
     */
    @GetMapping("/{id}")
    public RestResponse<PersonalVO> detail(@PathVariable Long id){
        PersonalVO personalVO =personalService.detail(id);
        return RestResponse.data(personalVO);
     }

    /**
     * 个贷业务订单新增
     */
    @PostMapping("")
    public synchronized RestResponse<WorkNodeCommit> save(@RequestBody Personal personal) {
        WorkNodeCommit ret = personalService.add(personal);
        return RestResponse.data(ret);
    }

    /**
     * 个贷业务订单更新
     */
    @PutMapping("")
    public RestResponse<Boolean> update(@RequestBody Personal personal) {
        Boolean ret = personalService.update(personal);
        return RestResponse.data(ret);
    }

    /**
     * 个贷业务订单删除
     */
    @DeleteMapping("/{id}")
    public RestResponse<Boolean> delete(@PathVariable Long id) {
        Boolean ret = personalService.delete(id);
        return RestResponse.data(ret);
    }

    /**
     * 分配外业人员
     * @param taskRecordDTO
     * @return
     */
    @FinishTask
    @PutMapping("/allot/outward")
    public RestResponse<Boolean> allotOutwardStaff(@RequestBody @Valid TaskRecordDTO<Personal> taskRecordDTO){
        Boolean ret = personalService.allotOutwardStaff(taskRecordDTO);
        return RestResponse.data(ret);
    }

}

