package com.dayou.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.dayou.annotation.IgnoreAuth;
import com.dayou.common.BaseEntity;
import com.dayou.entity.BusinessReply;
import com.dayou.entity.PersonalProduction;
import com.dayou.exception.ErrorCode;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.dayou.service.IPersonalProductionService;
import com.dayou.common.RestResponse;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

import static com.dayou.exception.ErrorCode.DEFAULTERROR;

/**
 * 个贷产品
 *
 * @author wucl
 * @since 2024-03-18
 */
@RestController
@RequestMapping("personalProduction")
@Slf4j
public class PersonalProductionController extends BaseController {
    @Autowired
    private IPersonalProductionService personalProductionService;


    /**
     * 个贷产品文档生成
     */
    @GetMapping("/{personalId}/{tag}")
    public RestResponse<Boolean> genFile(@PathVariable("personalId") Long personalId,@PathVariable("tag") Integer tag){
        Boolean ret = personalProductionService.genFile(personalId,tag);
        return RestResponse.data(ret);
     }

    /**
     * 产品包
     * @param targetId
     * @return
     */
    @GetMapping("/{targetId}")
     public RestResponse<List<PersonalProduction>> getProductions(@PathVariable("targetId") Long targetId){
        List<PersonalProduction> list = personalProductionService.list(new LambdaQueryWrapper<PersonalProduction>().eq(PersonalProduction::getTargetId, targetId).eq(BaseEntity::getDeleted, Boolean.FALSE));
        return RestResponse.data(list);
    }

    /**
     * 上传产品文档
     */
    @IgnoreAuth
    @PostMapping("/upload/{targetId}/{tag}")
    public RestResponse<Boolean> upload(MultipartFile file,@PathVariable("targetId") Long targetId,@PathVariable("tag") Integer tag) {
        try {
            // 上传文件路径
            String filePath = dfsConfig.getPath();
            Boolean ret = personalProductionService.upload(filePath, file,targetId,tag);
            return RestResponse.data(ret);
        } catch (Exception e) {
            log.error("upload error", e);
            ErrorCode.throwBusinessException(DEFAULTERROR);
        }

        return RestResponse.error();
    }

    /**
     * 移除文件
     * @param id
     * @return
     */
    @GetMapping("/remove/{id}")
    public RestResponse<Boolean> removeFile(@PathVariable("id") Long id){
        Boolean ret = personalProductionService.removeFile(id);
        return RestResponse.data(ret);
    }
}

