package com.dayou.controller;

import com.dayou.vo.MajorTaskDoneVO;
import com.dayou.vo.TaskTodoVO;
import com.dayou.vo.WorkTaskRecordVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.web.bind.annotation.RestController;
import com.dayou.service.workflow.IWorkTaskRecordService;
import com.dayou.entity.WorkTaskRecord;
import com.dayou.common.RestResponse;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 工作流任务记录
 *
 * @author wucl
 * @since 2023-10-17
 */
@RestController
@RequestMapping("workTaskRecord")
@Slf4j
public class WorkTaskRecordController extends BaseController {
    @Autowired
    private IWorkTaskRecordService workTaskRecordService;

    /**
    * 工作流任务记录列表
    */
    @GetMapping("")
    public RestResponse<Page<WorkTaskRecord>> page(WorkTaskRecord workTaskRecord, Page page){
        Page<WorkTaskRecord> pages=workTaskRecordService.selectPage(page,workTaskRecord);
        return RestResponse.data(pages);
    }

    /**
     * 工作流任务记录详情
     */
    @GetMapping("/{id}")
    public RestResponse<WorkTaskRecord> detail(@PathVariable Long id){
        WorkTaskRecord xWorkTaskRecord =workTaskRecordService.detail(id);
        return RestResponse.data(xWorkTaskRecord);
     }

    /**
     * 工作流任务记录新增
     */
    @PostMapping("")
    public RestResponse<Boolean> save(@RequestBody WorkTaskRecord workTaskRecord) {
        Boolean ret = workTaskRecordService.add(workTaskRecord);
        return RestResponse.data(ret);
    }

    /**
     * 工作流任务记录更新
     */
    @PutMapping("")
    public RestResponse<Boolean> update(@RequestBody WorkTaskRecord workTaskRecord) {
        Boolean ret = workTaskRecordService.update(workTaskRecord);
        return RestResponse.data(ret);
    }

    /**
     * 工作流任务记录删除
     */
    @DeleteMapping("/{id}")
    public RestResponse<Boolean> delete(@PathVariable Long id) {
        Boolean ret = workTaskRecordService.delete(id);
        return RestResponse.data(ret);
    }

    /**
     * 获取实例节点上的任务信息
     * @param instanceId
     * @return
     */
    @GetMapping("/instance/{instanceId}")
    public RestResponse<List<WorkTaskRecordVO>> getInstanceTask(@PathVariable("instanceId") Long instanceId){
        List<WorkTaskRecordVO> ret = workTaskRecordService.getInstanceTask(instanceId);
        return RestResponse.data(ret);
    }

    /**
     * 大中型业务我的待办
     * @param todoVO
     * @param page
     * @param keyword
     * @return
     */
    @GetMapping("/major/todo")
    public RestResponse<Page<TaskTodoVO>> taskTodoPage(TaskTodoVO todoVO, Page page,String keyword){
        Page<TaskTodoVO> ret = workTaskRecordService.majorTaskTodoPage(todoVO,page,keyword);
        return RestResponse.data(ret);
    }

    /**
     * 大中型业务我的已办
     * @param doneVO
     * @param page
     * @return
     */
    @GetMapping("/major/done")
    public RestResponse<Page<MajorTaskDoneVO>> taskDonePage(MajorTaskDoneVO doneVO, Page page,String keyword){
        Page<MajorTaskDoneVO> ret = workTaskRecordService.majorTaskDonePage(doneVO,page,keyword);
        return RestResponse.data(ret);
    }

}

