package com.dayou.service.impl;

import com.dayou.entity.FinanceClaim;
import com.dayou.entity.FinanceRealFund;
import com.dayou.exception.ErrorCode;
import com.dayou.mapper.FinanceClaimMapper;
import com.dayou.service.IFinanceClaimService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.service.IFinanceRealFundService;
import com.dayou.service.IProductionFundService;
import com.dayou.utils.LoginContext;
import com.dayou.vo.FinanceClaimVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.dayou.utils.ExcelUtil;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.math.BigDecimal;
import java.util.List;
import java.util.ArrayList;
import org.springframework.transaction.annotation.Transactional;
import com.dayou.enums.BatchTaskTypeEnum;

/**
 * <p>
 * 实收款认领 服务实现类
 * </p>
 *
 * @author wucl
 * @since 2023-12-08
 */
@Service
public class FinanceClaimServiceImpl extends ServiceImpl<FinanceClaimMapper, FinanceClaim> implements IFinanceClaimService {

    @Autowired
    private FinanceClaimMapper financeClaimMapper;

    @Autowired
    private IFinanceRealFundService financeRealFundService;

    @Autowired
    private IProductionFundService productionFundService;

    @Override
    @SuppressWarnings("unchecked")
    public Page<FinanceClaim> selectPage(Page page,FinanceClaim financeClaim){
        Page ret = this.page(page, new QueryWrapper<FinanceClaim>(financeClaim));
        return ret;
    }


    @Override
    public FinanceClaim detail(Long id){
        return this.getById(id);
    }

    @Transactional
    @Override
    public synchronized Boolean add(FinanceClaim financeClaim){
        BigDecimal notClaimAmount = financeRealFundService.getNotClaimAmount(financeClaim.getRealFundId());
        if (financeClaim.getClaimAmount().compareTo(notClaimAmount)<=0){
            financeClaim.setClaimId(LoginContext.getCurrentUserId());
            this.save(financeClaim);
            productionFundService.updateRealFundForProduction(financeClaim.getOrderFundId());
            return Boolean.TRUE;
        }else{
            ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR,"认领金额已超过实收款剩余金额，请检查！");
            return Boolean.FALSE;
        }
    }

    @Transactional
    @Override
    public Boolean update(FinanceClaim financeClaim){
        FinanceClaim xFinanceClaim = this.getById(financeClaim.getId());
        BigDecimal claimAmount = xFinanceClaim.getClaimAmount();
        BigDecimal notClaimAmount = financeRealFundService.getNotClaimAmount(financeClaim.getRealFundId());
        notClaimAmount = claimAmount.add(notClaimAmount);
        if (financeClaim.getClaimAmount().compareTo(notClaimAmount)<=0){
            financeClaim.setClaimId(LoginContext.getCurrentUserId());
            this.updateById(financeClaim);
            productionFundService.updateRealFundForProduction(xFinanceClaim.getOrderFundId());
            return Boolean.TRUE;
        }else{
            ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR,"认领金额已超过实收款剩余金额，请检查！");
            return Boolean.FALSE;
        }
    }

    @Transactional
    @Override
    public Boolean delete(Long id){
        //逻辑删除
        FinanceClaim xFinanceClaim = this.getById(id);
        this.removeById(id);
        productionFundService.updateRealFundForProduction(xFinanceClaim.getOrderFundId());
        return Boolean.TRUE;
    }

    @Override
    public List<FinanceClaimVO> getList(Long realFundId) {
        List<FinanceClaimVO> list = financeClaimMapper.getList(realFundId);
        return list;
    }

    @Override
    public BigDecimal getTotalClaimAmountByProductionFundId(Long id) {
        BigDecimal totalClaimAmount =  financeClaimMapper.getTotalClaimAmountByProductionFundId(id);
        return totalClaimAmount;
    }


}
