package com.dayou.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.dayou.common.BaseEntity;
import com.dayou.dto.TaskRecordDTO;
import com.dayou.dto.WorkNodeCommit;
import com.dayou.entity.GlobalConfig;
import com.dayou.entity.Personal;
import com.dayou.entity.PersonalTarget;
import com.dayou.entity.User;
import com.dayou.enums.CertificateEnum;
import com.dayou.enums.GlobalConfigEnum;
import com.dayou.enums.MainBusinessEnum;
import com.dayou.exception.ErrorCode;
import com.dayou.mapper.PersonalMapper;
import com.dayou.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.service.workflow.IWorkFlowService;
import com.dayou.vo.PersonalVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import java.util.List;
import org.springframework.transaction.annotation.Transactional;

import static com.dayou.common.Constants.PERSONAL_INWARD;

/**
 * <p>
 * 个贷业务订单 服务实现类
 * </p>
 *
 * @author wucl
 * @since 2024-03-01
 */
@Service
public class PersonalServiceImpl extends ServiceImpl<PersonalMapper, Personal> implements IPersonalService {

    @Autowired
    private PersonalMapper personalMapper;

    @Autowired
    private IBusinessNumberService businessNumberService;

    @Autowired
    private IWorkFlowService workFlowService;

    @Autowired
    private IUserService userService;
    
    @Autowired
    private IGlobalConfigService globalConfigService;

    @Autowired
    private IPersonalTargetService personalTargetService;

    @Override
    @SuppressWarnings("unchecked")
    public Page<PersonalVO> selectPage(Page page, PersonalVO personal){
        Page<PersonalVO> pages = personalMapper.xSelectPage(page,personal);
        return pages;
    }


    @Override
    public PersonalVO detail(Long id){
        return personalMapper.getDetail(id);
    }

    @Transactional
    @Override
    public WorkNodeCommit add(Personal personal){
        personal.setOrderId(businessNumberService.takeOrderNumber());
        //确定内业人员
        personal.setInwardStaff(nextInwardStaff());
        this.save(personal);
        //创建估价对象
        PersonalTarget pt = new PersonalTarget();
        pt.setPersonalId(personal.getId());
        pt.setLocation(personal.getLocation());
        personalTargetService.save(pt);
        //个贷下单
        return workFlowService.openingOrder(MainBusinessEnum.PERSONAL_BUSINESS,personal.getId(),personal.getOrderId(),personal.getRemark());
    }

    @Override
    public Boolean update(Personal personal){
        return  this.updateById(personal);
    }

    @Override
    public Boolean delete(Long id){
        //逻辑删除
        return this.removeById(id);
    }

    @Override
    public Boolean allotOutwardStaff(TaskRecordDTO<Personal> taskRecordDTO) {
        Personal personal = taskRecordDTO.getTaskData();
        return this.update(new LambdaUpdateWrapper<Personal>().set(Personal::getOutwardStaff,personal.getOutwardStaff()).eq(BaseEntity::getId,personal.getId()));
    }

    @Override
    public void updateCertificates(Long targetId, CertificateEnum cer) {
        Personal personal = personalMapper.getByTargetId(targetId);
        if (personal!=null && StrUtil.isNotEmpty(personal.getCredentials())){
            List<String> credentials = JSON.parseArray(personal.getCredentials(), String.class);
            if (!credentials.contains(cer.name())){
                credentials.add(cer.name());
                this.update(new LambdaUpdateWrapper<Personal>().set(Personal::getCredentials,JSON.toJSONString(credentials)).eq(BaseEntity::getId,personal.getId()));
            }
        }
    }

    private synchronized Long nextInwardStaff(){
        GlobalConfig inwardStaffRoll = globalConfigService.getGlobalConfig(GlobalConfigEnum.DEPARTMENT_TURNS.getCode(), GlobalConfigEnum.DepartmentTurns.INWARD_STAFF_ROLL.getCode());
        List<User> users = userService.simplePostName(PERSONAL_INWARD);
        if (CollectionUtil.isEmpty(users)){
            ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR,"未配置内业人员，请检查。");
        }
        Long nextStaffId = users.get(0).getId() ;
        if (StrUtil.isNotEmpty(inwardStaffRoll.getValue())){
            for (int i=0;i<users.size();i++){
                if (users.get(i).getId().equals(Long.valueOf(inwardStaffRoll.getValue()))){
                    if (i!=users.size()-1){
                        nextStaffId = users.get(i+1).getId();
                    }else {
                        nextStaffId = users.get(0).getId();
                    }
                }
            }
        }
        inwardStaffRoll.setValue(String.valueOf(nextStaffId));
        globalConfigService.update(inwardStaffRoll);
        return nextStaffId;
    }
}
