package com.dayou.service.impl;

import com.dayou.common.BaseEntity;
import com.dayou.dto.TaskRecordDTO;
import com.dayou.entity.MajorProduction;
import com.dayou.entity.ProductionFund;
import com.dayou.entity.ProductionOutWarehouse;
import com.dayou.enums.FinanceInvoiceState;
import com.dayou.enums.MainBusinessEnum;
import com.dayou.exception.ErrorCode;
import com.dayou.mapper.ProductionOutWarehouseMapper;
import com.dayou.service.IMajorProductionService;
import com.dayou.service.IProductionFundService;
import com.dayou.service.IProductionOutWarehouseService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.service.IUserService;
import com.dayou.utils.DateUtils;
import com.dayou.utils.LoginContext;
import com.dayou.vo.ProductionOutWarehouseApplyVO;
import org.apache.commons.collections.BagUtils;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.dayou.utils.ExcelUtil;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.time.LocalDateTime;
import java.util.List;
import java.util.ArrayList;
import org.springframework.transaction.annotation.Transactional;
import com.dayou.enums.BatchTaskTypeEnum;

/**
 * <p>
 * 产品提前出库申请 服务实现类
 * </p>
 *
 * @author wucl
 * @since 2023-12-12
 */
@Service
public class ProductionOutWarehouseServiceImpl extends ServiceImpl<ProductionOutWarehouseMapper, ProductionOutWarehouse> implements IProductionOutWarehouseService {

    @Autowired
    private ProductionOutWarehouseMapper outWarehouseMapper;

    @Autowired
    private IUserService userService;

    @Autowired
    private IMajorProductionService majorProductionService;

    @Override
    @SuppressWarnings("unchecked")
    public Page<ProductionOutWarehouse> selectPage(Page page,ProductionOutWarehouse productionOutWarehouse,String keyword){
        Page<ProductionOutWarehouse>  ret = outWarehouseMapper.getPage(page,productionOutWarehouse,keyword);
        return ret;
    }


    @Override
    public ProductionOutWarehouse detail(Long id){
        return this.getById(id);
    }

    @Override
    public Boolean add(TaskRecordDTO<ProductionOutWarehouse> taskRecordDTO){
        ProductionOutWarehouse productionOutWarehouse = taskRecordDTO.getTaskData();
        if (productionOutWarehouse.getBusinessType().equals(MainBusinessEnum.MAJOR_BUSINESS.name())){
            MajorProduction production = majorProductionService.getOne(new LambdaQueryWrapper<MajorProduction>().select(MajorProduction::getRepertoryState,MajorProduction::getReportNo)
                    .eq(BaseEntity::getId, productionOutWarehouse.getId())
                    .eq(MajorProduction::getDeleted, Boolean.FALSE));
            if (production!=null && production.getRepertoryState()!=null && !production.getRepertoryState()){
                productionOutWarehouse.setApplyId(LoginContext.getCurrentUserId());
                productionOutWarehouse.setProductionId(productionOutWarehouse.getId());
                productionOutWarehouse.setReportNo(production.getReportNo());
                //获取财务审核人
                return  this.save(productionOutWarehouse);
            }
        }
        ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR,"产品不在库存中，还未入库或则已经出库。请确认。");
        return Boolean.FALSE;
    }

    @Override
    public Boolean update(TaskRecordDTO<ProductionOutWarehouse> taskRecordDTO){
        ProductionOutWarehouse productionOutWarehouse = taskRecordDTO.getTaskData();
        ProductionOutWarehouse outWarehouse = this.getById(productionOutWarehouse.getId());
        if (outWarehouse.getProductionOut()){
            ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR,"已进行了出库操作，出库申请无法修改。");
            return Boolean.FALSE;
        }
//        productionOutWarehouse.setCheckerId(LoginContext.getCurrentUserId());
//        productionOutWarehouse.setCheckTime(DateUtils.getNowDate());
        if (productionOutWarehouse.getCheckType()){
            productionOutWarehouse.setFinanceCheckerId(LoginContext.getCurrentUserId());
            productionOutWarehouse.setFinanceCheckTime(DateUtils.getNowDate());
            productionOutWarehouse.setFinanceCheckState(productionOutWarehouse.getState());
            productionOutWarehouse.setFinanceReply(productionOutWarehouse.getReply());
        }else{
            productionOutWarehouse.setDepartmentCheckerId(LoginContext.getCurrentUserId());
            productionOutWarehouse.setDepartmentCheckTime(DateUtils.getNowDate());
            productionOutWarehouse.setDepartmentCheckState(productionOutWarehouse.getState());
            productionOutWarehouse.setDepartmentReply(productionOutWarehouse.getReply());
        }
        return  this.updateById(productionOutWarehouse);
    }

    @Override
    public Boolean delete(Long id){
        //逻辑删除
        return this.removeById(id);
    }

    @Override
    public List<ProductionOutWarehouseApplyVO> getList(Long id) {
        List<ProductionOutWarehouseApplyVO> list = outWarehouseMapper.getList(id);
        return list;
    }

    @Override
    public Boolean cancel(Long id) {
        ProductionOutWarehouse outWarehouse = this.getById(id);
        if (outWarehouse.getProductionOut()){
            ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR,"已进行了出库操作，出库申请无法撤销。");
        }
        boolean ret = this.removeById(id);
        return ret;
    }

    @Override
    public List<ProductionOutWarehouseApplyVO> getListByBizTypeProductionId(String businessType, Long productionId) {
       return outWarehouseMapper.getListByBizTypeProductionId(businessType,productionId);
    }
}
