package com.dayou.workflow.config;


import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.dayou.common.BaseEntity;
import com.dayou.dto.HandlerPermissionDTO;
import com.dayou.dto.WorkNodeCommit;
import com.dayou.entity.*;
import com.dayou.enums.MainBusinessEnum;
import com.dayou.enums.workflow.HandlerPermissionEnum;
import com.dayou.enums.workflow.NodeLogEnum;
import com.dayou.exception.ErrorCode;
import com.dayou.mapper.DepartmentMapper;
import com.dayou.mapper.MajorMapper;
import com.dayou.mapper.PersonalMapper;
import com.dayou.mapper.WorkTaskRecordMapper;
import com.dayou.service.IUserPostService;
import com.dayou.service.IUserService;
import com.dayou.service.workflow.IWorkFlowLogService;
import com.dayou.service.workflow.IWorkFlowNodeInstanceService;
import com.dayou.service.workflow.IWorkTaskRecordService;
import com.dayou.utils.LoginContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;

import static com.dayou.enums.workflow.NodeStateEnum.*;

/**
 * 类说明：
 *
 * @author: wucl
 * @since: 2023/11/1
 * created with IntelliJ IDEA.
 */
@Component
public class WorkNodeProcessable {

    @Autowired
    private IWorkFlowNodeInstanceService workFlowNodeInstanceService;
    @Autowired
    private IWorkTaskRecordService workTaskRecordService;
    @Autowired
    private IWorkFlowLogService workFlowLogService;
    @Autowired
    private WorkTaskRecordMapper workTaskRecordMapper;



    /**
     * 前进
     * @param commit
     * @return
     */
    public boolean forward(WorkNodeCommit commit){
        WorkFlowNodeInstance instanceNode = workFlowNodeInstanceService.getById(commit.getInstanceNodeId());

        //节点控制前置处理。
        //当任务不需要当前节点处理人产生业务数据的时候，则把任务状态改为已完成。
        List<Long> notBusinessHandleTaskId = workTaskRecordMapper.getNotBusinessHandleTaskId(instanceNode.getId());
        if (CollectionUtil.isNotEmpty(notBusinessHandleTaskId)){
            return workTaskRecordService.update(new LambdaUpdateWrapper<WorkTaskRecord>().set(WorkTaskRecord::getFinished,Boolean.TRUE)
                    .set(WorkTaskRecord::getFinishTime, LocalDateTime.now()).in(BaseEntity::getId,notBusinessHandleTaskId));
        }

        // 校验相同节点是否已创建
//        WorkFlowNodeInstance nextInstanceNode = workFlowNodeInstanceService.getOne(new LambdaQueryWrapper<WorkFlowNodeInstance>()
//                .eq(WorkFlowNodeInstance::getFlowId, instanceNode.getFlowId()).eq(WorkFlowNodeInstance::getBusinessId, instanceNode.getBusinessId())
//                .eq(BaseEntity::getDeleted, Boolean.FALSE));

        return instanceNode!=null;
    }

    /**
     * 跳过
     * @param commit
     * @return
     */
    @Transactional
    public boolean skip(WorkNodeCommit commit){
        return Boolean.FALSE;
    }

    /**
     * 退回
     * @param commit
     * @return
     */
    @Transactional
    public boolean goBack(WorkNodeCommit commit){
        Long instanceNodeId = commit.getInstanceNodeId();
        WorkFlowNodeInstance currentNode = workFlowNodeInstanceService.getById(instanceNodeId);

        //将上一个节点状态改为PENDING
        workFlowNodeInstanceService.update(new LambdaUpdateWrapper<WorkFlowNodeInstance>().set(WorkFlowNodeInstance::getState,PENDING)
                .eq(WorkFlowNodeInstance::getBusinessId,currentNode.getBusinessId()).eq(WorkFlowNodeInstance::getFlowId,currentNode.getFlowId())
                .eq(StrUtil.isNotEmpty(commit.getBusinessSubId()),WorkFlowNodeInstance::getBusinessSubId,commit.getBusinessSubId())
                .eq(StrUtil.isNotEmpty(commit.getBusinessMinId()),WorkFlowNodeInstance::getBusinessMinId,commit.getBusinessMinId())
                .eq(WorkFlowNodeInstance::getSequence,(currentNode.getSequence()-1)).eq(BaseEntity::getDeleted,Boolean.FALSE));

        //将上一个节点的任务状态修改为未完成
        WorkFlowNodeInstance preNode = workFlowNodeInstanceService.getOne(new LambdaQueryWrapper<WorkFlowNodeInstance>().select(BaseEntity::getId)
                .eq(WorkFlowNodeInstance::getFlowId, currentNode.getFlowId())
                .eq(WorkFlowNodeInstance::getBusinessId, currentNode.getBusinessId()).eq(WorkFlowNodeInstance::getSequence, (currentNode.getSequence() - 1))
                .eq(StrUtil.isNotEmpty(commit.getBusinessSubId()),WorkFlowNodeInstance::getBusinessSubId,commit.getBusinessSubId())
                .eq(StrUtil.isNotEmpty(commit.getBusinessMinId()),WorkFlowNodeInstance::getBusinessMinId,commit.getBusinessMinId())
                .eq(BaseEntity::getDeleted, Boolean.FALSE));
        if (preNode!=null){
            workTaskRecordService.update(new LambdaUpdateWrapper<WorkTaskRecord>().set(WorkTaskRecord::getFinished,Boolean.FALSE).eq(WorkTaskRecord::getInstanceId,preNode.getId()));
        }

        //删除当前节点
        workFlowNodeInstanceService.delete(instanceNodeId);

        //删除当前节点的待处理的任务记录
        workTaskRecordService.remove(new LambdaQueryWrapper<WorkTaskRecord>().eq(WorkTaskRecord::getInstanceId,instanceNodeId));

        //添加日志
        WorkFlowLog log = new WorkFlowLog();
                log.setBusinessId(currentNode.getBusinessId());
                log.setBusinessType(currentNode.getBusinessType());
                log.setHandingTime(new Date().getTime() - currentNode.getCreated().getTime());
                log.setHandlerId(LoginContext.getCurrentUserId());
                log.setComments(commit.getComments());
                log.setState(NodeLogEnum.REVERSE.name());
                log.setInstanceId(commit.getInstanceNodeId());
        return workFlowLogService.add(log);

    }

    /**
     * 重置
     * @param commit
     * @return
     */
    public boolean reset(WorkNodeCommit commit){
        //todo
        return Boolean.FALSE;
    }

    /**
     * 终止
     * @param commit
     * @return
     */
    @Transactional
    public boolean ending(WorkNodeCommit commit){
        WorkFlowNodeInstance currentNode = workFlowNodeInstanceService.getById(commit.getInstanceNodeId());

        //只需将当前节点设置为终止状态。
        workFlowNodeInstanceService.update(new LambdaUpdateWrapper<WorkFlowNodeInstance>().set(WorkFlowNodeInstance::getState,CLOSED)
                        .set(WorkFlowNodeInstance::getComment,commit.getComments())
                .eq(WorkFlowNodeInstance::getId,commit.getInstanceNodeId()));

        //添加日志
        WorkFlowLog log = new WorkFlowLog();
                    log.setBusinessId(currentNode.getBusinessId());
                    log.setBusinessType(currentNode.getBusinessType());
                    log.setHandingTime(new Date().getTime() - currentNode.getCreated().getTime());
                    log.setHandlerId(LoginContext.getCurrentUserId());
                    log.setComments(commit.getComments());
                    log.setState(NodeLogEnum.TERMINATE.name());
                    log.setInstanceId(commit.getInstanceNodeId());
        return workFlowLogService.add(log);
    }


    public Long searchHandlerId(WorkFlowNodeInstance instanceNode, HandlerPermissionDTO handlerPermissionDTO){
        return 0L;
    }
}
