package com.dayou.workflow.handler;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.dayou.common.BaseEntity;
import com.dayou.dto.HandlerPermissionDTO;
import com.dayou.dto.WorkNodeCommit;
import com.dayou.entity.*;
import com.dayou.enums.workflow.HandlerPermissionEnum;
import com.dayou.enums.workflow.NodeLogEnum;
import com.dayou.mapper.*;
import com.dayou.service.IUserPostService;
import com.dayou.service.IUserService;
import com.dayou.service.workflow.IWorkFlowLogService;
import com.dayou.service.workflow.IWorkFlowNodeInstanceService;
import com.dayou.utils.LoginContext;
import com.dayou.workflow.annotation.WorkflowControl;
import com.dayou.workflow.config.WorkNodeProcessable;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

import static com.dayou.enums.workflow.NodeStateEnum.SKIP;
import static com.dayou.enums.workflow.WorkflowNodeEnum.*;

/**
 * 类说明：
 *
 * @author: wucl
 * @since: 2023/11/1
 * created with IntelliJ IDEA.
 */
@Component
public class MajorNodeHandler extends WorkNodeProcessable {

    @Autowired
    private IWorkFlowNodeInstanceService workFlowNodeInstanceService;
    @Autowired
    private WorkFlowNodeInstanceMapper workFlowNodeInstanceMapper;
    @Autowired
    private IWorkFlowLogService workFlowLogService;
    @Autowired
    private MajorProductionMapper majorProductionMapper;
    @Autowired
    private MajorTargetMapper majorTargetMapper;
    @Autowired
    private MajorMapper majorMapper;
    @Autowired
    private DepartmentMapper departmentMapper;
    @Autowired
    private IUserPostService userPostService;


    @Override
    @WorkflowControl
    public boolean forward(WorkNodeCommit workNodeCommit) {
         return super.forward(workNodeCommit);
    }

    @Override
    @WorkflowControl
    public boolean skip(WorkNodeCommit workNodeCommit){
        WorkFlowNodeInstance instanceNode = workFlowNodeInstanceService.getById(workNodeCommit.getInstanceNodeId());
         workFlowNodeInstanceService.update(new LambdaUpdateWrapper<WorkFlowNodeInstance>().set(WorkFlowNodeInstance::getState,SKIP)
                .set(WorkFlowNodeInstance::getComment,workNodeCommit.getComments())
                .eq(WorkFlowNodeInstance::getId,workNodeCommit.getInstanceNodeId()));
        //添加日志
        WorkFlowLog log = new WorkFlowLog();
                    log.setBusinessId(instanceNode.getBusinessId());
                    log.setBusinessType(instanceNode.getBusinessType());
                    log.setHandingTime(new Date().getTime() - instanceNode.getCreated().getTime());
                    log.setHandlerId(LoginContext.getCurrentUserId());
                    log.setComments(workNodeCommit.getComments());
                    log.setState(NodeLogEnum.SKIP.name());
                    log.setInstanceId(workNodeCommit.getInstanceNodeId());
        return workFlowLogService.add(log);
    }

    @Override
    public boolean goBack(WorkNodeCommit commit){
        Long instanceNodeId = commit.getInstanceNodeId();
        WorkFlowNodeInstance currentNode = workFlowNodeInstanceMapper.getInstanceNodeInfoById(instanceNodeId);
        //如果是撰写节点（意见书，报告，复评函）退回。需删除产品业务信息
        List<String> writeNodes = Arrays.asList(new String[]{WRITE_STATEMENT.name(), WRITE_REPORT.name(), WRITE_LETTER.name()});
        if (writeNodes.contains(currentNode.getNodeCode()) && commit.getCleanReportNo()!=null && commit.getCleanReportNo()){
            //是否取号？
            List<String> reportNos =  majorTargetMapper.getSubReportNo(currentNode.getBusinessId());
            if (CollectionUtil.isNotEmpty(reportNos)){
                //清除子号
                majorTargetMapper.cleanSubReportNo(currentNode.getBusinessId());
                //清除产品信息
                majorProductionMapper.delete(new LambdaQueryWrapper<MajorProduction>().eq(MajorProduction::getMajorId,currentNode.getBusinessId()));
            }
        }
        return super.goBack(commit);
    }

    @Override
    public Long searchHandlerId(WorkFlowNodeInstance instanceNode, HandlerPermissionDTO handlerPermissionDTO){
        HandlerPermissionEnum handlerPermissionEnum = handlerPermissionDTO.getName();
        Major major = majorMapper.selectOne(new LambdaQueryWrapper<Major>()
                        .select(Major::getDepartmentId,Major::getClientManagerId,Major::getPrincipalId).eq(Major::getId,instanceNode.getBusinessId()));
        switch (handlerPermissionEnum){
            case EMPLOYEE:
                return handlerPermissionDTO.getPowerId();
            case POST:
                Long handlerId = userPostService.getUniqueHandlerByPostForWorkNode(handlerPermissionDTO.getPowerId());
                return handlerId==null?
                        departmentMapper.selectOne(new LambdaQueryWrapper<Department>().select(Department::getLeaderId).eq(BaseEntity::getId, major.getDepartmentId())).getLeaderId()
                        :handlerId;
            case DEPARTMENT_LEADER:
                return departmentMapper.selectOne(new LambdaQueryWrapper<Department>().select(Department::getLeaderId).eq(BaseEntity::getId, major.getDepartmentId())).getLeaderId();
            case CLIENT_MANAGER:
            case INITIATOR:
                return major.getClientManagerId();
            case PRINCIPAL:
                return major.getPrincipalId();
            case SYSTEM:
                //暂无程序处理逻辑
                return 0L;
            default:
                //实在找不到审批人 交由部门领导处理
                return departmentMapper.selectOne(new LambdaQueryWrapper<Department>().select(Department::getLeaderId).eq(BaseEntity::getId, major.getDepartmentId())).getLeaderId();
        }
    }
}
