package com.dayou.workflow.handler;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.dayou.common.BaseEntity;
import com.dayou.dto.HandlerPermissionDTO;
import com.dayou.dto.WorkNodeCommit;
import com.dayou.entity.Department;
import com.dayou.entity.Personal;
import com.dayou.entity.WorkFlowNodeInstance;
import com.dayou.enums.workflow.HandlerPermissionEnum;
import com.dayou.mapper.DepartmentMapper;
import com.dayou.mapper.PersonalMapper;
import com.dayou.service.IUserPostService;
import com.dayou.workflow.annotation.WorkflowControl;
import com.dayou.workflow.config.WorkNodeProcessable;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import static com.dayou.common.Constants.PERSONAL_DEPARTMENT;

/**
 * 类说明：
 *
 * @author: wucl
 * @since: 2024/3/4
 * created with IntelliJ IDEA.
 */
@Component
public class PersonalNodeHandler extends WorkNodeProcessable {

    @Autowired
    private PersonalMapper personalMapper;

    @Autowired
    private DepartmentMapper departmentMapper;

    @Autowired
    private IUserPostService userPostService;




    @Override
    @WorkflowControl
    public boolean forward(WorkNodeCommit workNodeCommit) {
       return super.forward(workNodeCommit);
    }

    @Override
    public Long searchHandlerId(WorkFlowNodeInstance instanceNode, HandlerPermissionDTO handlerPermissionDTO){
        HandlerPermissionEnum handlerPermissionEnum = handlerPermissionDTO.getName();
        Personal personal = personalMapper.selectOne(new LambdaQueryWrapper<Personal>().select(Personal::getClientManagerId, Personal::getInwardStaff, Personal::getOutwardStaff, Personal::getPricingStaff)
                .eq(BaseEntity::getId, instanceNode.getBusinessId()));
        switch (handlerPermissionEnum){
            case EMPLOYEE:
                return handlerPermissionDTO.getPowerId();
            case POST:
                Long handlerId = userPostService.getUniqueHandlerByPostForWorkNode(handlerPermissionDTO.getPowerId());
                return handlerId==null?
                        departmentMapper.selectOne(new LambdaQueryWrapper<Department>().select(Department::getLeaderId).eq(Department::getName,PERSONAL_DEPARTMENT)).getLeaderId()
                        :handlerId;
            case DEPARTMENT_LEADER:
                return departmentMapper.selectOne(new LambdaQueryWrapper<Department>().select(Department::getLeaderId).eq(Department::getName,PERSONAL_DEPARTMENT)).getLeaderId();
            case OUTWARD_STAFF:
                return personal.getOutwardStaff();
            case INWARD_STAFF:
                return personal.getInwardStaff();
            case PRICING_STAFF:
                return personal.getPricingStaff();
            case SYSTEM:
                //暂无程序处理逻辑
                return 0L;
            default:
                //实在找不到审批人 交由部门领导处理
                return departmentMapper.selectOne(new LambdaQueryWrapper<Department>().select(Department::getLeaderId).eq(Department::getName,PERSONAL_DEPARTMENT)).getLeaderId();
        }
    }
}
